import 'dart:convert';
import 'dart:developer';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mono_widget/src/html%20_files/mono_connect_html.dart';
import 'package:webview_flutter/webview_flutter.dart';

class FlutterMonoConnectWidget extends StatefulWidget {
  const FlutterMonoConnectWidget({
    Key? key,
    required this.apiKey,
    this.onSuccess,
    this.onClosed,
    this.error,
  }) : super(key: key);

  // your mono api key.
  final String apiKey;

  /// callback when transaction succeeds
  final Function(String code)? onSuccess;

  /// callback when user clicks the close button the on mono's page
  final Function()? onClosed;

  /// An overlay widget to display over webview if page fails to load
  final Widget? error;

  @override
  State<FlutterMonoConnectWidget> createState() =>
      _FlutterMonoConnectWidgetState();
}

class _FlutterMonoConnectWidgetState extends State<FlutterMonoConnectWidget> {
  late WebViewController _webViewController;
  // final baseUrl = 'https://connect.withmono.com/?key=';
  bool isLoading = false;
  bool hasError = false;

  @override
  void initState() {
    if (Platform.isAndroid) WebView.platform = SurfaceAndroidWebView();
    super.initState();
  }

  // String getUrl() => '$baseUrl${getMonoConnectHtml(widget.apiKey)}';

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        if (widget.onClosed != null) widget.onClosed!();
        return true;
      },
      child: SafeArea(
        child: Stack(
          children: [
            Container(
              // margin: EdgeInsets.only(top: 30),
              decoration:
                  BoxDecoration(border: Border.all(color: Colors.transparent)),
              child: WebView(
                // initialUrl: getUrl(),

                javascriptMode: JavascriptMode.unrestricted,
                onWebViewCreated: (WebViewController webViewController) async {
                  _webViewController = webViewController;
                  await webViewController
                      .loadHtmlString(getMonoConnectHtml(widget.apiKey));
                },
                onPageStarted: (String url) {
                  setState(() {
                    isLoading = true;
                    hasError = false;
                  });
                },
                javascriptChannels: <JavascriptChannel>{
                  _monoJavascriptChannel(context),
                },
                // gestureRecognizers: Set()
                //   ..add(Factory<TapGestureRecognizer>(
                //       () => TapGestureRecognizer()
                //         ..onTapDown = (tap) {
                //           SystemChannels.textInput.invokeMethod(
                //               'TextInput.hide'); //This will hide keyboard ontapdown
                //         })),
                debuggingEnabled: kDebugMode,
                onWebResourceError: (err) async {
                  isLoading = false;
                  setState(() {
                    hasError = true;
                  });
                },
                onPageFinished: (String url) async => setState(() {
                  isLoading = false;
                  log(url);
                }),
                onProgress: (int progress) {
                  log('WebView is loading (progress : $progress%)');
                },
              ),
            ),
            if (isLoading) const Center(child: CircularProgressIndicator()),
            // if (hasError) widget.error ?? _error
          ],
        ),
      ),
    );
  }

  JavascriptChannel _monoJavascriptChannel(BuildContext context) {
    return JavascriptChannel(
      name: 'MonoClientInterface',
      onMessageReceived: (JavascriptMessage message) {
        // if (kDebugMode) log('MonoClientInterface, ${message.message}');

        var res = json.decode(message.message);
        if (kDebugMode) {
          log('MonoClientInterface, ${(res as Map<String, dynamic>)}');
        }
        handleResponse(res as Map<String, dynamic>);
      },
    );
  }

  void handleResponse(Map<String, dynamic>? body) {
    String? key = body!['type'];
    if (key != null) {
      switch (key) {

        // todo: change the event name
        // case 'mono.connect.widget.account_linked':
        case 'mono.modal.linked':
          var response = body['response'];
          if (response == null) return;
          var code = response['code'];
          if (widget.onSuccess != null) widget.onSuccess!(code);
          if (mounted) Navigator.of(context).pop(code);
          break;
        // case 'mono.connect.widget.closed':
        case 'mono.modal.closed':
          if (widget.onClosed != null) widget.onClosed!();
          if (mounted) Navigator.of(context).pop();
          break;
        default:
      }
    }
  }
}
