import 'dart:convert';
import 'dart:developer';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mono_widget/src/html%20_files/mono_make_payment.htlm.dart';
import 'package:webview_flutter/webview_flutter.dart';

class FlutterMonoPaymentWidget extends StatefulWidget {
  const FlutterMonoPaymentWidget({
    Key? key,
    required this.apiKey,
    required this.amountInKobo,
    required this.description,
    required this.transactionReference,
    required this.reference,
    this.onSuccess,
    this.onClosed,
    this.error,
  }) : super(key: key);

  // your mono api key.
  final String apiKey;

  // reference for the transaction
  final String reference;

  // amount to pay
  final int amountInKobo;

  final String description;

  // unique id to reference the transaction
  final String transactionReference;

  /// callback when transaction succeeds
  final Function(Map<String, dynamic> data)? onSuccess;

  /// callback when user clicks the close button the on mono's page
  final Function()? onClosed;

  /// An overlay widget to display over webview if page fails to load
  final Widget? error;

  @override
  State<FlutterMonoPaymentWidget> createState() =>
      _FlutterMonoPaymentWidgetState();
}

class _FlutterMonoPaymentWidgetState extends State<FlutterMonoPaymentWidget> {
  late WebViewController _webViewController;
  bool isLoading = false;
  bool hasError = false;

  @override
  void initState() {
    if (Platform.isAndroid) WebView.platform = SurfaceAndroidWebView();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        if (widget.onClosed != null) widget.onClosed!();
        return true;
      },
      child: SafeArea(
        child: SingleChildScrollView(
          child: SizedBox(
            height: MediaQuery.of(context).size.height * 0.98,
            child: Stack(
              children: [
                Container(
                  decoration: BoxDecoration(
                      border: Border.all(color: Colors.transparent)),
                  child: WebView(
                    javascriptMode: JavascriptMode.unrestricted,
                    onWebViewCreated:
                        (WebViewController webViewController) async {
                      _webViewController = webViewController;
                      await webViewController.loadHtmlString(
                        getMonoPayHtml(
                          key: widget.apiKey,
                          amountInKobo: widget.amountInKobo,
                          description: widget.description,
                          transactionReference: widget.transactionReference,
                          reference: widget.reference,
                        ),
                      );
                    },
                    onPageStarted: (String url) {
                      setState(() {
                        isLoading = true;
                        hasError = false;
                      });
                    },
                    javascriptChannels: <JavascriptChannel>{
                      _monoJavascriptChannel(context),
                    },
                    debuggingEnabled: kDebugMode,
                    onWebResourceError: (err) async {
                      isLoading = false;
                      setState(() {
                        hasError = true;
                      });
                    },
                    onPageFinished: (String url) async => setState(() {
                      isLoading = false;
                      log(url);
                    }),
                    onProgress: (int progress) {
                      log('WebView is loading (progress : $progress%)');
                    },
                  ),
                ),
                if (isLoading) const Center(child: CircularProgressIndicator()),
                // if (hasError) widget.error ?? _error
              ],
            ),
          ),
        ),
      ),
    );
  }

  JavascriptChannel _monoJavascriptChannel(BuildContext context) {
    return JavascriptChannel(
      name: 'MonoClientInterface',
      onMessageReceived: (JavascriptMessage message) {
        if (kDebugMode) log('MonoClientInterface, ${message.message}');

        var res = json.decode(message.message);
        if (kDebugMode) {
          // log('MonoClientInterface, ${(res as Map<String, dynamic>)}');
        }
        handleResponse(res as Map<String, dynamic>);
      },
    );
  }

  void handleResponse(Map<String, dynamic>? body) {
    String? key = body!['type'];
    if (key != null) {
      switch (key) {
        // case 'mono.connect.widget.account_linked':
        // case 'mono.modal.linked':
        //   var response = body['response'];
        //   if (response == null) return;
        //   var code = response['code'];
        //   if (widget.onSuccess != null) widget.onSuccess!(code);
        //   if (mounted) Navigator.of(context).pop(code);
        //   break;
        // case 'mono.connect.widget.closed':
        case 'mono.modal.closed':
          if (widget.onClosed != null) widget.onClosed!();
          if (mounted) Navigator.of(context).pop();
          break;

        case 'mono.connect.widget.account_linked':
          var response = body['data'];
          if (widget.onSuccess != null) widget.onSuccess!(response);
          if (mounted) Navigator.of(context).pop();

          break;
        default:
      }
    }
  }
}
